#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Configuration
import os
import sys

sys.path.append(os.getcwd())
from connlp.preprocess import EnglishTokenizer
from connlp.visualize import Visualizer


## Visualize
# Word Network
def test_word_network(docs):
    tokenizer = EnglishTokenizer()
    tokenized_docs = []
    for sent in docs:
        tokenized_sent = tokenizer.tokenize(text=sent)
        tokenized_docs.append(tokenized_sent)

    visualizer = Visualizer()
    visualizer.network(docs=tokenized_docs, show=True)


## Run
if __name__ == '__main__':
    docs = ['I am a boy', 'She is a girl']
    test_word_network(docs=docs)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Configuration
import os
import sys

sys.path.append(os.getcwd())
from connlp.preprocess import EnglishTokenizer
from connlp.embedding import Vectorizer
tokenizer = EnglishTokenizer()
vectorizer = Vectorizer()


## Embedding
# Word2Vec
def test_word2vec_development(docs):
    tokenized_docs = [tokenizer.tokenize(text=doc) for doc in docs]
    w2v_model = vectorizer.word2vec(docs=tokenized_docs)

    print(w2v_model.wv['boy'])
    print(w2v_model.wv.most_similar('boy', topn=3))
    return w2v_model

def test_word2vec_udpate(w2v_model, new_docs):
    tokenized_new_docs = [tokenizer.tokenize(text=doc) for doc in new_docs]
    w2v_model_updated = vectorizer.word2vec_update(w2v_model=w2v_model, new_docs=tokenized_new_docs)

    print(w2v_model_updated.wv['man'])
    print(w2v_model_updated.wv.most_similar('man', topn=3))
    return w2v_model_updated

# Doc2Vec
def test_doc2vec_development(docs):
    tagged_docs = [(idx, tokenizer.tokenize(text=doc)) for idx, doc in enumerate(docs)]
    d2v_model = vectorizer.doc2vec(tagged_docs=tagged_docs)

    test_doc = ['My', 'name', 'is', 'Peter']
    print(d2v_model.infer_vector(doc_words=test_doc))


## Run
if __name__ == '__main__':
    docs = ['I am a boy', 'He is a boy', 'She is a girl']
    new_docs = ['Tom is a man', 'Sally is not a boy']
    
    w2v_model = test_word2vec_development(docs=docs)
    w2v_model_updated = test_word2vec_udpate(w2v_model=w2v_model, new_docs=new_docs)

    d2v_model = test_doc2vec_development(docs=docs)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Configuration
import re


class Normalizer:
    '''
    A class used to normalize text with predetermined character replacement rules.

    Attributes
    ----------
    do_lower : Boolean
        | To lower the text or not. (default : True)
    do_marking : Boolean
        | To mark several tokens to predetermined entities or not. (default : False)
        | The predetermined entities include "LINK", "REF", and "NUM".

    Methods
    -------
    normalize
        | Normalizes the input text and returns a clean text.
    '''

    def __init__(self, do_lower=True, do_marking=False, **kwargs):
        self.do_lower = do_lower
        self.do_marking = do_marking

    def __remove_trash_char(self, text):
        text = re.sub('[^ \'\?\./0-9a-zA-Zㄱ-힣\n]', '', text)

        # Remove debris from format conversion
        text = text.replace(' / ', '/')
        text = re.sub('\.+\.', ' ', text)
        text = text.replace('\\\\', '\\').replace('\\r\\n', '')
        text = text.replace('\n', '  ')
        text = re.sub('\. ', '  ', text)
        text = re.sub('\s+\s', ' ', text).strip()

        # Lower the text
        if self.do_lower:
            text = text.lower()
        else:
            pass
        
        # Remove endmarks
        if text.endswith('\.'):
            text = text[:-1]
        else:
            pass
            
        return text

    def __marking(self, text):
        for i, w in enumerate(sent):
            if re.match('www.', str(w)):
                sent[i] = 'LINK'
            elif re.search('\d+\d\.\d+', str(w)):
                sent[i] = 'REF'
            elif re.match('\d', str(w)):
                sent[i] = 'NUM'
            else:
                continue

        return sent

    def normalize(self, text):
        '''
        A method to normalize the input text.

        Attributes
        ----------
        text : str
            | An input text in str type.
        '''

        if self.do_marking:
            text = self.__marking(text)
        else:
            pass
            
        return self.__remove_trash_char(text)

    ## TODO: udpate character replacement rules
    # def update_char_list(self, input_char_list, verbose=False):
    #     cnt_before = len(self.remain_char_list)
    #     self.remain_char_list = list(set(self.remain_char_list.extend(input_char_list)))
    #     cnt_after = len(self.remain_char_list)

    #     if verbose:
    #         print('|Update remain_char_list: [{}] chars --> [{}] chars'.format(cnt_before, cnt_after))


class EnglishTokenizer:
    '''
    A class to tokenize an English sentence.

    Attributes
    ----------
    n : int
        | ngram size. (default : 1)

    Methods
    -------
    tokenize
        | tokenizes the input sentence.
    '''

    def __init__(self, **kwargs):
        self.n = kwargs.get('n', 1) # ngram size

    def tokenize(self, text):
        '''
        A method to tokenize the input text.

        Attributes
        ----------
        text : str
            | An input text in str type.
        '''

        result = [w for w in re.split(' |  |\n', text) if w]
        return result

    ## TODO: split based on ngrams


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Configuration
import itertools
from collections import defaultdict

import networkx as nx
import matplotlib.pyplot as plt


class CombCounter:
    '''
    A class used to count word combinations from a list of tokenized sentences.

    Attributes
    ----------
    docs : list
        | A list of tokenized words.
    normalize : Boolean
        | if True, the counter normalizes the weights based on the distance between words within the sentence.
        | elif False, it counts the frequency.
    
    Methods
    -------
    simple_dict
        | Counts word combinations from docs.
        | Returns a dictionary of which the key is 'WORD[A]__WORD[B]' and the value is occurred frequency.
    '''

    def __init__(self, docs, **kwargs):
        self.docs = docs
        self.normalize = kwargs.get('normalize', False)

    def simple_dict(self):
        count_dict = defaultdict(int)
        for sent in self.docs:
            for pair in itertools.combinations(sent, 2):
                count_dict['__'.join(pair)] += 1

        return count_dict

    ## TODO: counting method with normalized weights based on the distance between words within the sentence.


class Visualizer:
    '''
    A class used to visualize text mining results.

    Methods
    -------
    network(docs=None, show=False, fig_width=10, fig_height=8, dpi=300, arrows=False, node_size=10, font_size=8, width=0.6, edge_color='grey', node_color='purple', with_labels=False, facecolor='white', alpha=0, edgecolor='white'))
        | Returns a word network.
    '''

    def network(self, docs, show=False, fig_width=10, fig_height=8, dpi=300, arrows=False, node_size=10, font_size=8, width=0.6, edge_color='grey', node_color='purple', with_labels=False, facecolor='white', alpha=0, edgecolor='white'):
        '''
        A method to return word network

        Copyright (C) 2004-2021, NetworkX Developers
        Aric Hagberg <hagberg@lanl.gov>
        Dan Schult <dschult@colgate.edu>
        Pieter Swart <swart@lanl.gov>
        All rights reserved.

        Attributes
        ----------
        docs : list
            | A list of tokenized words.
        show : Boolean
            | Whether show the figure or not (default : False).
        
        NOTE: other attributes follow the original documentation of NetworkX.
        '''

        data = CombCounter(docs=docs)
        graph = nx.DiGraph()
        for key, score in data.simple_dict().items():
            node_from, node_to = key.split('__')
            graph.add_edge(node_from, node_to, weight=score)
        pos = nx.spring_layout(graph, k=0.08)

        fig, ax = plt.subplots(figsize=(fig_width, fig_height), dpi=dpi)
        nx.draw_networkx(
            graph, pos,
            arrows=arrows,
            node_size=node_size,
            font_size=font_size,
            width=width,
            edge_color=edge_color,
            node_color=node_color,
            with_labels=with_labels,
            ax=ax)

        for key, value in pos.items():
            ax.text(x=value[0],
                    y=value[1]+0.025,
                    s=key,
                    bbox=dict(facecolor=facecolor, alpha=alpha, edgecolor=edgecolor),
                    horizontalalignment='center',
                    fontsize=font_size)

        if show:
            plt.show()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Configuration
import os
import sys

sys.path.append(os.getcwd())
from connlp.preprocess import EnglishTokenizer, Normalizer


## Preprocess
# Normalizer
def test_normalizer(docs):
    normalizer = Normalizer()
    for sent in docs:
        normalized_sent = normalizer.normalize(text=sent)
        print(normalized_sent)

# English Tokenizer
def test_english_tokenizer(docs):
    tokenizer = EnglishTokenizer()
    for sent in docs:
        tokenized_sent = tokenizer.tokenize(text=sent)
        print(tokenized_sent)


## Run
if __name__ == '__main__':
    docs = ['I am a boy!', 'She is a girl']

    test_normalizer(docs=docs)
    test_english_tokenizer(docs=docs)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Configuration
from gensim.models import Word2Vec, Doc2Vec
from gensim.models.doc2vec import TaggedDocument
from sklearn.feature_extraction.text import TfidfVectorizer


class Vectorizer:
    def __init__(self, **kwargs):
        pass

    ## TODO
    # def tfidf(self, docs):
    #     vectorizer = TfidfVectorizer()
    #     tfidf_model = vectorizer.fit_transform(docs)
    #     return tfidf_model

    ## TODO: verbose
    def word2vec(self, docs, verbose=True, **kwargs):
        '''
        A method to develop a new Word2Vec model.
        Refer to: https://radimrehurek.com/gensim/models/word2vec.html

        Attributes
        ----------
        docs : list
            | a list of tokenized docs.
        parameters : dict
            | a dictionary of user-determined hyperparameters
        '''

        parameters = kwargs.get('parameters', {})
        model = Word2Vec(
            sentences=docs,
            size=parameters.get('size', 100),
            window=parameters.get('window', 5),
            min_count=parameters.get('min_count', 0),
            sg=parameters.get('sg', 0),
            hs=parameters.get('hs', 0),
            negative=parameters.get('negative', 5),
            ns_exponent=parameters.get('ns_exponent', 0.75),
            iter=parameters.get('iter', 5)
        )
        return model

    def word2vec_update(self, w2v_model, new_docs, verbose=True):
        '''
        A method to update an already developed Word2Vec model with new docs.
        Refer to: https://radimrehurek.com/gensim/models/word2vec.html

        Attributes
        ----------
        w2v_model : gensim.models.word2vec.Word2Vec
            | an already developed Word2Vec model.
        new_docs : list
            | a list of tokenized docs to update the Word2Vec model.
        '''

        w2v_model.min_count = 0
        w2v_model.build_vocab(sentences=new_docs, update=True)
        w2v_model.train(sentences=new_docs, total_examples=w2v_model.corpus_count, epochs=w2v_model.iter)
        return w2v_model

    ## TODO: verbose
    def doc2vec(self, tagged_docs, verbose=True, **kwargs):
        '''
        A method to develop a new Doc2Vec model.
        REfer to: https://radimrehurek.com/gensim/models/doc2vec.html

        Attributes
        ----------
        tagged_docs : list
            | A list of tuples that include tag and tokenized sentence.
            | E.g.) [(tag1, [w1, w2, ...]), (tag2, [w3, w4, ...]), (tag3, [w5, ...])]
        parameters : dict
            | a dictionary of user-determined hyperparameters
        '''

        parameters = kwargs.get('parameters', {})
        docs_for_d2v = [TaggedDocument(words=doc, tags=[tag]) for tag, doc in tagged_docs]
        d2v_model = Doc2Vec(
            documents=docs_for_d2v,
            vector_size=parameters.get('vector_size', 100),
            window=parameters.get('window', 5),
            min_count=parameters.get('min_count', 0),
            dm=parameters.get('dm', 1),
            negative=parameters.get('negative', 5),
            epochs=parameters.get('epochs', 5),
            dbow_words=parameters.get('dbow_words', 1)
        )
        return d2v_model


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Configuration
import os
import pandas as pd


def makedir(fpath):
    '''
    A method to make directory for the given file path.

    Attributes
    ----------
    fpath : str
        | A file path.
    '''

    if fpath.endswith('/'):
        os.makedirs(fpath, exist_ok=True)
    else:
        os.makedirs('/'.join(fpath.split('/')[:-1]), exist_ok=True)


def export_xlsx(data, fpath, index=False, verbose=False):
    '''
    A method to export excel of the given data.

    Attributes
    ----------
    data : dict
        | A dictionary of which column lengths are the same.
    fpath : str
        | The file path to export the xlsx.
    index : Boolean
        | Write row names or not. (default : False)
    verbose : Boolean
        | Print fpath or not. (default : False)

    NOTE: other attributes follow the original documentation of pandas.DataFrame.
    '''

    if type(data) != 'pandas.core.frame.DataFrame':
        data = pd.DataFrame(data)

    makedir(fpath)
    writer = pd.ExcelWriter(fpath)
    data.to_excel(writer, "Sheet1", index=index)
    writer.save()

    if verbose:
        print("Saved data as: {}".format(fpath))


def f1_score(p, r):
    '''
    A method to calculate f1 score.

    Attributes
    ----------
    p : float
        | precision (None-zero)
    r : float
        | recall (None-zero)
    '''

    if p != 0 or r != 0:
        return (2*p*r)/(p+r)
    else:
        return 0



def accuracy(tp, tn, fp, fn):
    '''
    A method to calculate accuracy

    Attributes
    ----------
    tp : int
        | true positive
    tn : int
        | true negative
    fp : int
        | false positive
    fn : int
        | false negative
    '''

    return (tp+tn)/(tp+tn+fp+fn)


from setuptools import setup, find_packages

setup(
    name='connlp',
    version='0.0.7',
    license='',
    author='Seonghyeon Boris Moon',
    author_email='boris.moon514@gmail.com',
    description='A bunch of python codes to analyze text data in the construction industry. Mainly reconstitute the pre-exist python libraries for Natural Language Processing (NLP)',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/blank54/connlp.git',
    packages=find_packages(),
    classifiers=[],
)